;  Input:  inner_radius - radius of hole at center
;          outer_radius - radius at center of teeth
;          width - width of gear
;          teeth - number of teeth
;          tooth_depth - depth of tooth


Function make_gear(inner_radius#,outer_radius#,width#,teeth%,tooth_depth#)

Local i%
Local r0#, r1#, r2#
Local angle#, da#
Local u#, v#, ln#

  r0 = inner_radius
  r1 = outer_radius - tooth_depth / 2.0
  r2 = outer_radius + tooth_depth / 2.0

  da = 2.0 * 180 / teeth / 4.0

Local m=CreateMesh()
Local s=CreateSurface(m)

;make front face
For i = 0 To teeth-1
	angle = i * 2.0 * 180 / teeth
	AddVertex (s,r1 * Cos(angle), r1 * Sin(angle), width * 0.5)
	AddVertex (s,r0 * Cos(angle), r0 * Sin(angle), width * 0.5)
	AddVertex (s,r1 * Cos(angle + 3 * da), r1 * Sin(angle + 3 * da), width * 0.5)
Next

For i = 0 To teeth-2 
	w=i*3
	AddTriangle (s,4+w,2+w,3+w)
	AddTriangle (s,1+w,2+w,4+w)
	AddTriangle (s,0+w,2+w,1+w)
Next
	w=i*3
	AddTriangle (s,1,w+2,0)
	AddTriangle (s,1,1+w,w+2)
	AddTriangle (s,0+w,2+w,1+w)

;make front sides of teeth
For i = 0 To teeth-1
	angle = i * 2.0 * 180 / teeth
    AddVertex (s,r2 * Cos(angle + da), r2 * Sin(angle + da), width * 0.5)
    AddVertex (s,r2 * Cos(angle + 2 * da), r2 * Sin(angle + 2 * da), width * 0.5)
Next

w=-3
For i = 0 To teeth-1 
g=i*2+teeth*3
w=w+3
AddTriangle (s,0+w,0+g,2+w)
AddTriangle (s,0+g,1+g,2+w)
Next

;make back face
For i = 0 To teeth-1
	angle = i * 2.0 * 180 / teeth
	AddVertex (s,r1 * Cos(angle), r1 * Sin(angle), -width * 0.5)
	AddVertex (s,r0 * Cos(angle), r0 * Sin(angle), -width * 0.5)
	AddVertex (s,r1 * Cos(angle + 3 * da), r1 * Sin(angle + 3 * da), -width * 0.5)
Next

For i = 0 To teeth-2 
	w=teeth*5+i*3
	AddTriangle (s,4+w,3+w,2+w)
	AddTriangle (s,1+w,4+w,2+w)
	AddTriangle (s,0+w,1+w,2+w)
Next
	w=teeth*5+i*3
	AddTriangle (s,teeth*5+1,teeth*5,w+2)
	AddTriangle (s,teeth*5+1,w+2,1+w)
	AddTriangle (s,0+w,1+w,2+w)

;make back sides of teeth
For i = 0 To teeth-1
	angle = i * 2.0 * 180 / teeth
    AddVertex (s,r2 * Cos(angle + da), r2 * Sin(angle + da), -width * 0.5)
    AddVertex (s,r2 * Cos(angle + 2 * da), r2 * Sin(angle + 2 * da), -width * 0.5)
Next

w=teeth*5-3
For i = 0 To teeth-1 
g=teeth*5+i*2+teeth*3
w=w+3
AddTriangle (s,0+w,2+w,0+g)
AddTriangle (s,0+g,2+w,1+g)
Next

;fill sides
Local t3=teeth*3
Local t5=teeth*5
Local t8=teeth*8

w=-3
g=-2
For i = 0 To teeth-1
w=w+3:g=g+2
	AddTriangle (s,0+w,t5+w,t8+g)
	AddTriangle (s,0+w,t8+g,t3+g)

	AddTriangle (s,t3+g,t8+g,t8+1+g)
	AddTriangle (s,t3+g,t8+1+g,t3+1+g)

	AddTriangle (s,2+w,t3+1+g,t8+1+g)
	AddTriangle (s,2+w,t8+1+g,t5+2+w)
	
	If i<teeth-1
	AddTriangle (s,2+w,t5+2+w,t5+3+w)
	AddTriangle (s,2+w,t5+3+w,3+w)
	EndIf
Next

AddTriangle (s,0,t8-1,t5)
AddTriangle (s,0,t3-1,t8-1)

w=-3
For i = 0 To teeth-2
w=w+3
	AddTriangle (s,1+w,4+w,t5+1+w)
	AddTriangle (s,4+w,t5+4+w,t5+1+w)
Next

AddTriangle (s,1,t8-2,t3-2)
AddTriangle (s,1,t5+1,t8-2)

UpdateNormals m
Return m

End Function



;EXAMPLE-----------------

Graphics3D 640,480,32,2
c=CreateCamera()
l=CreateLight()
PositionEntity c,0,0,-19

;inner_radius#,outer_radius#,width#,teeth%,tooth_depth#
g1=make_gear(1.0, 4.0, 1.0, 20, 0.7)
EntityColor g1,255,0,0
PositionEntity g1,-3,-2,0
RotateEntity g1,0,0,0

g2=make_gear(0.5, 2.0, 2.0, 10, 0.7)
EntityColor g2,255,255,0
PositionEntity g2,3.1,-2,0
RotateEntity g2,0,0,-9

g3=make_gear(1.3, 2.0, 0.5, 10, 0.7)
EntityColor g3,0,255,0
PositionEntity g3,-3.1,4.2,0
RotateEntity g3,0,0,-25


g4=make_gear(1.0, 2, 0.5, 10, 0.7)
EntityColor g4,0,255,255
PositionEntity g4,-3.1,-8,0
RotateEntity g4,0,0,-25

g5=make_gear(1.0, 4.0, 1.0, 20, 0.7)
EntityColor g5,255,0,255
PositionEntity g5,9.5,-2,0
RotateEntity g5,0,0,0


ang#=0

While Not KeyHit(1)


ang#=ang#+1.0
If ang=360 ang=0

RotateEntity g1,0,0,ang
RotateEntity g2,0,0,-2*ang-9
RotateEntity g3,0,0,-2*ang-25

RotateEntity g4,0,0,-2*ang-25

RotateEntity g5,0,0,ang

RenderWorld()
If KeyHit (17)
w=1-w
WireFrame w
EndIf

Flip

Wend

End
